import logging
import numpy as np
from abc import ABC, abstractmethod

from tensorflow.keras.preprocessing.text import text_to_word_sequence

from promethist.common.utils import make_logger_name

logger = logging.getLogger(make_logger_name(__name__))


class EmbeddingBase(ABC):
    """
    Base class for embedding algorithms.
    This class needs to be derived.
    """

    def __init__(self, language, config):
        super().__init__()
        self.language = language
        self.padding = config["maxSequenceLength"]
        self.dim = -1
        self.alg = ''
        self.oov_vectors = {}

    def texts_to_embeddings(self, texts_list):
        """

        :param texts_list: Array of strings
        :return:
        """
        tokens_list = [text_to_word_sequence(text) for text in texts_list]
        return self.tokens_to_embeddings(tokens_list)

    @abstractmethod
    def tokens_to_embeddings(self, tokens_list):
        """

        :param tokens_list: Array of tokens
        :return:
        """
        pass

    def oov_vector(self, word, norm=30.0):
        """
        Function for handling out-of-vocabulary words.
        Creates a random embedding and stores it for reuse with the same OOV word.
        :param word: OOV word
        :param norm: norm of the resulting vector (weight in sentence)
        :return: OOV word embedding
        """

        if word in self.oov_vectors:
            return self.oov_vectors[word]

        vec = np.random.rand(self.dim) - 0.5
        vec /= np.linalg.norm(vec) / norm

        self.oov_vectors[word] = vec
        return vec