import json
import mlflow
import logging
from abc import ABC, abstractmethod

from promethist.common.utils import make_logger_name

logger = logging.getLogger(make_logger_name(__name__))


class AbstractKerasWrapper(ABC):
    def __init__(self, _id, language, config, vectorizer):
        self._id = _id
        self.language = language
        self.config = config

        self.vectorizer = vectorizer # tokens to vectors
        self.model = None
        self._label2idx = None
        self.idx2label = None

    @property
    def label2idx(self):
        return self._label2idx

    @label2idx.setter
    def label2idx(self, label_dict):
        self._label2idx = label_dict
        self.idx2label = {self.label2idx[k]: k for k in self.label2idx}

    def get_config_value(self, key):
        value = self.config.get(key, None)
        if value is None:
            logger.debug("Key {} is not present in config".format(key))
        return value


class SequenceAbstractAnnotator(AbstractKerasWrapper):

    def __init__(self, _id, language, model_data, mapping, config, vectorizer):
        super().__init__(_id=_id, language=language, config=config, vectorizer=vectorizer)
        self.model_data = model_data
        self.mapping = mapping

    def annotate(self, query):
        logger.debug("Query: {}".format(query))
        if "tokens" not in query:
            raise AttributeError("Query is not tokenized")
        prediction = self._annotate(query["tokens"])

        for i in range(len(query["tokens"])):
            if prediction[i] == "O":
                continue

            if "classes" in query["tokens"][i]:
                if any([1 if x["type"] == self.get_annotation_name() else 0 for x in query["tokens"][i]["classes"]]):
                    query["tokens"][i]["classes"].append({"type": self.get_annotation_name(), "name": prediction[i]})
                else:
                    raise RuntimeError("The query has been already annotated by type {}.".format(self.get_annotation_name()))
            else:
                query["tokens"][i]["classes"] = []
                query["tokens"][i]["classes"].append({"type": self.get_annotation_name(), "name": prediction[i]})

        return query

    @abstractmethod
    def get_annotation_name(self):
        raise NotImplementedError("You need to override this method.")

    @abstractmethod
    def _annotate(self, query):
        """
        :param query:
        :return: prediction (after idx2label)
        """
        raise NotImplementedError("You need to override this method.")

    @abstractmethod
    def load_model(self):
        raise NotImplementedError("You need to override this method.")


class SequenceAbstractTrainer(AbstractKerasWrapper):

    def __init__(self, _id, language, config, vectorizer):
        super().__init__(_id=_id, language=language, config=config, vectorizer=vectorizer)
        mlflow.set_tracking_uri(config["mlflow"]["url"])
        mlflow.set_experiment(config["mlflow"]["projectName"])

    @abstractmethod
    def save_model(self, persistence):
        raise NotImplementedError("You need to override this method.")

    @abstractmethod
    def _train(self, train_json, valid_json):
        """
        :param train_json:
        :return: history, params
        """
        raise NotImplementedError("You need to override this method.")

    def train(self, train_data, valid_data=None, test_data=None):
        """

        :param test_data:
        :param valid_data:
        :param train_data:
        :return:
        """
        history, params = self._train(train_data, valid_data)

        for k, v in params.items():
            if isinstance(v, dict):
                for k2, v2 in v.items():
                    mlflow.log_param(k + "." + k2, v2)
            else:
                mlflow.log_param(k, v)

        try:
            if test_data:
                acc, f1, recall, precision = self._evaluate(test_json=test_data, debug=False)
            else:
                acc, f1, recall, precision = self._evaluate(test_json=json.load(open(self.get_config_value("testJson"), "r")), debug=False)

            mlflow.log_metric("accuracy", acc)
            mlflow.log_metric("f1", f1)
            mlflow.log_metric("recall", recall)
            mlflow.log_metric("precision", precision)
        except FileNotFoundError:
            logger.debug("Cannot open test json: {}".format(self.get_config_value("testJson")))

    def _evaluate(self, test_json, debug):
        """

        :param test_json:
        :return: precision, recall, f1, support
        """
        raise NotImplementedError("You need to override this method.")
