# -*- coding: utf-8 -*-
"""
    library.utils
    ~~~~~~~~~~~~~~~~~

    Utility functions used throughout the project.
"""

import json
import datetime
import re

from os import getenv, path
from promethist.common.config import CONFIG


def format_brackets(token):
    """
    Formats bracket tokens.
    :param token: string token
    :return: formatted token
    """

    if token == '-LRB-':
        token = '('
    elif token == '-RRB-':
        token = ')'
    elif token == '-RSB-':
        token = ']'
    elif token == '-LSB-':
        token = '['
    elif token == '-LCB-':
        token = '{'
    elif token == '-RCB-':
        token = '}'
    return token


def tokenize(tokenizer, sentence, to_lower=True):
    """
    Tokenize sentence.
    :param tokenizer: a tokenizer implementing the NLTK tokenizer interfaces
    :param sentence: a string to be tokenized
    :param to_lower: lowercase input
    :return: tokenized string
    """

    sentence = sentence.strip()
    sentence = ' '.join([format_brackets(x) for x in tokenizer.tokenize(sentence)])
    if to_lower:
        sentence = sentence.lower()
    sentence = re.sub(r"((www\.[^\s]+)|(https?://[^\s]+)|(http?://[^\s]+))", "<url>", sentence)
    sentence = re.sub(r"(@[^\s]+)", "<user>", sentence)
    filter(lambda word: r" " not in word, sentence)
    return sentence


def get_subwords(word, nmin=3, nmax=6):
    """
    Get a list of n-grams in <word> with n between <nmin> and <nmax>.
    :param word: input word
    :param nmin: lower bound on n
    :param nmax: upper bound on n
    :return: list of n-grams in <word>
    """

    word = f'<{word}>'
    return [word[i:j] for i in range(len(word))
            for j in range(i + nmin, 1 + min(i + nmax, len(word)))]


class DateTimeEncoder(json.JSONEncoder):
    def default(self, o):
        if isinstance(o, datetime.datetime):
            return o.strftime("%Y-%m-%dT%H:%M:%S%z")


def make_logger_name(file_name, logger_name='flask.app'):
    """ Join the default logger name with a module name. """
    return logger_name + "." + file_name


def get_memory_limit(logger):
    """
    Get current memor limit via docker setting or configuration value.
    :param logger: logger
    :return: current memory limit <int>
    """
    try:
        with open("/sys/fs/cgroup/memory/memory.limit_in_bytes") as memory:
            memory_value = memory.read()
            memory_limit = int(int(memory_value) * 0.7)
            logger.info("Container memory value: %s", memory_value)
            logger.info("Container memory limit: %s", memory_limit)
    except FileNotFoundError:
        memory_limit = int(CONFIG[getenv("APP", "app")]["memoryLimit"])
        logger.info("Memory limit from application configuration: %s", memory_limit)
    return memory_limit


def init_swagger_host(module_name, spec_dir):
    """
    Replace swagger service host via configuration.
    :param module_name: module name
    :param spec_dir: swagger specification directory
    """

    default_host = str(CONFIG[module_name]['host']) + ':' + str(CONFIG[module_name]["port"])
    try:
        service_host = CONFIG[module_name]['serviceHost']
    except KeyError:
        service_host = default_host

    # change service host in swagger.yaml
    if service_host != default_host:
        swagger_path = path.join(spec_dir, CONFIG["api"]["name"])
        with open(swagger_path) as swagger:
            data = swagger.read()
        with open(swagger_path, 'w') as swagger:
            swagger.write(data.replace(default_host, service_host))
